@@ -296,14 +296,14 @@ def v2i(x, v):
 
 
 ###################################################################################
-def enc_apl(bng, x, met, skipo=True):
+def enc_apl(bng, x, met):
     if isinstance(bng, list):
         m = len(bng)
     else:
         m = 1
     N = x.shape[0]
     xenc = np.empty((N, 0))
-    cenc = []
+    ss = []
     for i in range(m):
         if isinstance(x, pd.DataFrame):
             xi = c_(x.iloc[:, i].values)
@@ -318,34 +318,34 @@ def enc_apl(bng, x, met, skipo=True):
             mi = len(bns); cname_e = []; j = 0
         else:
             mi = 1
-        xe = nans((N, mi - skipo))
+        cnd_st = [None]*mi
+        xe = nans((N, mi))
         for h in range(1, len(bns) + 1):
-            if skipo and bns[h]['type'] == 'Other': continue
-            dv = b2dv(bns[h], xi, xtp)
-            if met == 'my':
-                xe[dv] = bns[h]['my'][1];       cname_e = cname + '_encMY'
-            elif met == 'woe':
-                xe[dv] = bns[h]['int'][1];      cname_e = cname + '_encWOE'
-            elif met == 'int':
-                xe[dv] = h - 1;                 cname_e = cname + '_encINT'
-            elif met == 'dv':
-                xe[:, j] = dv.flatten(); cname_e = cname_e + [cname + '_encDV' + str(j + 1)]; j += 1
+            if met == 'dv' and not bns[h]['n'] > 0: np.delete(xe, h, axis=1); continue
+            dv = b2dv(bns[h], xi, xtp)==1
+            if met == 'my':      xe[dv] = bns[h]['my'];  cname_e = cname + '_encMY'
+            elif met == 'woe':   xe[dv] = bns[h]['woe'];  cname_e = cname + '_encWOE'
+            elif met == 'int':   xe[dv] = h - 1;          cname_e = cname + '_encINT'
+            elif met == 'dv':    xe[:, j] = dv.flatten()*1; cname_e = cname_e + [cname + '_encDV' + str(j + 1)]; j += 1
         if met == 'dv': xe = xe.astype(int)
-
+        print('isnan(xe): ', np.any(np.isnan(xe)))
+        if np.any(np.isnan(xe)):
+            prinr("DA EBA MAMA TI BEEEEEEEEEEEEE!!!!!!!!!!!!!!!!!")
         xenc = np.hstack((xenc, xe))
-        cenc = cenc + cname_e
+        if isinstance(cname_e, str): ss.append(cname_e)
+        else:                        ss = ss + cname_e
 
-    return xenc, cenc
+    return xenc, ss
 
 
 ###################################################################################
 def inv_distr(DistrType, pval, df1, df2, x1, x2, tol):
     ITMAX = 100;
     EPS = 3.0e-7
-    a = x1;
-    b = x2;
-    c = x2;
-    d = 0;
+    a = x1
+    b = x2
+    c = x2
+    d = 0
     e = 0
     cnd = DistrType == 'Fisher'
     if cnd:
@@ -361,16 +361,16 @@ def inv_distr(DistrType, pval, df1, df2, x1, x2, tol):
     it_num = range(0, ITMAX)
     for i in it_num:
         if fb > 0 and fc > 0 or fb < 0 and fc < 0:
-            c = a;
-            fc = fa;
-            e = b - a;
+            c = a
+            fc = fa
+            e = b - a
             d = e
         if abs(fc) < abs(fb):
-            a = b;
-            b = c;
-            c = a;
-            fa = fb;
-            fb = fc;
+            a = b
+            b = c
+            c = a
+            fa = fb
+            fb = fc
             fc = fa
         tol1 = 2 * EPS * abs(b) + 0.5 * tol
         xm = 0.5 * (c - b)
@@ -383,7 +383,7 @@ def inv_distr(DistrType, pval, df1, df2, x1, x2, tol):
                 p = 2 * xm * s
                 q = 1 - s
             else:
-                q = fa / fc;
+                q = fa / fc
                 r = fb / fc
                 p = s * (2 * xm * q * (q - r) - (b - a) * (r - 1))
                 q = (q - 1) * (r - 1) * (s - 1)
@@ -394,10 +394,10 @@ def inv_distr(DistrType, pval, df1, df2, x1, x2, tol):
             min2 = abs(e * q)
             cnd2 = min1 < min2
             if 2 * p < (cnd2 * min1 + int(not cnd2) * min2):
-                e = d;
+                e = d
                 d = p / q
             else:
-                d = xm;
+                d = xm
                 e = d
         else:
             d = xm;
@@ -646,7 +646,7 @@ def reduce_bns2(bns, ytp, md, mar):
 
 ###################################################################################
 def sbng(bng, md=5, skip_mos=['m', 'o', 'sv'], met_dist='manh', met='co1y', nmin_uy=[50, 50], mar=np.inf, pt=0.05, ba=0,
-         maxprm=20000, dGBInd=5, tolmindGBInd=5, mt=np.nan, minB=50, tolminB=20):
+         maxprm=20000, dGBInd=5, tolmindGBInd=5, mt=np.nan, minB=50, tolminB=20, stats=True, dsp=False):
     # bng1 = copy.deepcopy(bng)
     # del bng
     # bng = bng1
@@ -677,8 +677,8 @@ def sbng(bng, md=5, skip_mos=['m', 'o', 'sv'], met_dist='manh', met='co1y', nmin
         elif met == 'co1y':
             gd = nmin_uy[0]
             bd = nmin_uy[1]
-            bns, D = spih_sb_co1y(bns, xtp, ytp, md, gd, bd, mar, pt, ba, dGBInd, tolmindGBInd, mt, minB, tolminB,
-                                  maxprm)
+            bns, D = spih_sb_co1y(bns, xtp, ytp, md, gd, bd, mar, pt, ba, dGBInd, tolmindGBInd, mt, minB, tolminB, maxprm)
+
         if bns is None:
             sb[i]['bns'] = {}
         else:
@@ -689,7 +689,9 @@ def sbng(bng, md=5, skip_mos=['m', 'o', 'sv'], met_dist='manh', met='co1y', nmin
         sb[i]['ytp'] = ytp
         sb[i]['st'] = {}
         sb[i]['st']['D'] = D
-        print('Run SB: ', i, '/', m, cname, ',      time: ', toc(0))
+        if stats: st, sb[i]['bns'] = stbng('sb', sb[i]['bns'], xtp, ytp)
+
+        if dsp: print('Run SB: ', i + 1, '/', m, cname, ',      time: ', toc(0))
 
     if m == 1:
         return sb[0]
@@ -1050,19 +1052,18 @@ def stbng(met, bns, xtp, ytp, x=np.empty((0, 0)), y=np.empty((0, 0)), w=np.empty
     if bns == {}: s = []; return st
     r = len(ytp)
     m = len(bns)
-    mn = 0  # number of Normal bins
-    for h in range(1, m + 1):
-        if bns[h]['type'] == 'Normal': mn += 1
+    mn = 0  # number of not empty bins
     Nw = 0
-    for i in range(1, m + 1): Nw = Nw + bns[i]['nw']
-    #    for i in range(1,len(bns)+1): print(bns[i]['nw'])
+    for h in range(1, m + 1):
+        if bns[h]['n'] > 0: mn += 1
+        Nw = Nw + bns[h]['nw']
     n = nans((mn,))
     nw = nans((mn,))
     hh = 0
-    for h in range(m):
-        if bns[h + 1]['type'] == 'Normal':
-            n[hh] = bns[h + 1]['n']
-            nw[hh] = bns[h + 1]['nw']
+    for h in range(1, m + 1):
+        if bns[h]['n'] > 0:
+            n[hh] = bns[h]['n']
+            nw[hh] = bns[h]['nw']
             hh += 1
 
     for i in range(r):
@@ -1072,21 +1073,20 @@ def stbng(met, bns, xtp, ytp, x=np.empty((0, 0)), y=np.empty((0, 0)), w=np.empty
             sy = nans((mn, nyi))
             sy2 = nans((mn, nyi))
             hh = 0
-            for h in range(m):
+            for h in range(1, m + 1):
                 # if ytpi == 'nom' or ytpi == 'ord': continue # todo: should collect sy and sy2 also for categorical variables - len(sy) = len(sy2) = len(nyi). Currently len(bns[h + 1]['y'][i + 1]) varies for different h, but should be nyi.
-                if len(bns[h + 1]['y']) > 1 and bns[h + 1]['type'] == 'Normal':
+                if bns[h]['n'] > 0:
                     sy[hh] = bns[h + 1]['y'][i + 1]
                     sy2[hh] = bns[h + 1]['y2'][i + 1]
                     hh += 1
-
         else:
             sy = nans((mn,))
             sy2 = nans((mn,))
             hh = 0
-            for h in range(m):
-                if bns[h + 1]['type'] == 'Normal':
-                    sy[hh] = bns[h + 1]['y'][i + 1][0]
-                    sy2[hh] = bns[h + 1]['y2'][i + 1][0]
+            for h in range(1, m + 1):
+                if bns[h]['n'] > 0:
+                    sy[hh] = bns[h]['y'][i + 1][0]
+                    sy2[hh] = bns[h]['y2'][i + 1][0]
                     hh += 1
             nw = c_(nw)
             sy = c_(sy)
@@ -1101,8 +1101,10 @@ def stbng(met, bns, xtp, ytp, x=np.empty((0, 0)), y=np.empty((0, 0)), w=np.empty
             s['pnw'] = nw / Nw * 100
             s['py0'] = np.dot(sy, np.linalg.pinv(Sy[np.newaxis])) * 100
             s['py1'] = np.dot(syb, np.linalg.pinv([Nw - Sy])) * 100
-            s['cp0'] = np.cumsum(s['py0'])
-            s['cp1'] = np.cumsum(s['py1'])
+            temp = s['py0']; temp[np.isnan(temp)] = 0
+            s['cp0'] = np.cumsum(temp)
+            temp = s['py0']; temp[np.isnan(temp)] = 0
+            s['cp1'] = np.cumsum(temp)
             nw1 = nw
             nw1[nw < 1e-12] = 1e-12
             s['my0'] = np.array(syb / nw1)
@@ -1113,11 +1115,11 @@ def stbng(met, bns, xtp, ytp, x=np.empty((0, 0)), y=np.empty((0, 0)), w=np.empty
 
             s['odds'] = sy / syb
             s['Gini'] = gini(sy, syb)
-            # s['WoE'] = woe(sy, syb)
+            s['WoE'] = woe(sy, syb)
             s['KS'] = ks(sy, syb)
             s['Vin'] = vinf(sy, syb)
             s['D'] = dind(sy, syb)
-            if m == 2: s['twoing'] = twng(sy, syb)
+            #if m == 2: s['twoing'] = twng(sy, syb)
             s['Chi2'] = chi2(sy, syb)
         elif ytpi == 'num':
             s['sy'] = sy
@@ -1126,14 +1128,14 @@ def stbng(met, bns, xtp, ytp, x=np.empty((0, 0)), y=np.empty((0, 0)), w=np.empty
         else:
             s = []
         st[i] = s
-        # for i in range(m):
-        #     if 'WoE' in s:  bns[i+1]['woe'] = s['WoE'][i]
-        #     else:           bns[i+1]['woe'] = None
-        #     if 'my' in s:   bns[i+1]['my'] = s['my1'][i]
-        #     else:           bns[i+1]['my'] = None
+        hh = 0
+        for h in range(1, m + 1):
+            if 'WoE' in s and bns[h]['n'] > 0:  bns[h]['woe'] = s['WoE'][hh, 0]
+            else:                               bns[h]['woe'] = None
+            if 'my1' in s and bns[h]['n'] > 0:  bns[h]['my'] = s['my1'][hh, 0]
+            else:                               bns[h]['my'] = None
+            if bns[h]['n'] > 0: hh += 1
     return st, bns
-
-
 ###################################################################################
 def stbngs(s, sy, nw):
     Np, md = s.shape
@@ -1159,7 +1161,7 @@ def stbngs(s, sy, nw):
 ###################################################################################
 # def ubng(x, w, par, y=np.empty((0,0))):
 def ubng(x, xtp, w, md=100, prcd=1, nmin=50, met='enr', sv=[], xi_order=[], y=np.empty((0, 0)), ytp='', yi=[],
-         cnames=None):
+         cnames=None, stats=False, dsp=False):
     m = x.shape[1]
     if cnames is None: cnames = ['var_' + str(i) for i in range(m)]
     ub = [None] * m
@@ -1210,7 +1212,7 @@ def ubng(x, xtp, w, md=100, prcd=1, nmin=50, met='enr', sv=[], xi_order=[], y=np
         ub[i]['cname'] = cnames[i]
         ub[i]['xtp'] = xtpi
         ub[i]['ytp'] = ytp
-        print('Run UB: ', i, '/', m, cnames[i], ',      time: ', toc(0))
+        print('Run UB: ', i + 1, '/', m, cnames[i], ',      time: ', toc(0))
 
     if m == 1:
         return ub[0]
