@@ -268,15 +268,19 @@ def gbi(g, b):
     G = np.tile(g.sum(axis=0), (m, 1))
     B = np.tile(b.sum(axis=0), (m, 1))
     b[b < 1e-12] = 1e-12
-    gbo = g / b;
+
+    import warnings
+    warnings.filterwarnings("ignore", category=RuntimeWarning)
+
+    gbo = g / b
     GBO = G / B
     gbo = gbo.flatten()
     GBO = GBO.flatten()
     gbi = np.zeros(len(gbo))
-    ind = gbo >= GBO;
+    ind = gbo >= GBO
     gbi[ind] = gbo[ind] / GBO[ind] * 100
     gbo[gbo < 1e-12] = 1e-12
-    ind = gbo < GBO;
+    ind = gbo < GBO
     gbi[ind] = -GBO[ind] / gbo[ind] * 100
     gbi = gbi.reshape(m, n)
     gbi1 = gbi + 200 * (gbi < 0).astype(int)
@@ -316,6 +320,7 @@ def gcf(a, x, delta=1e-7, maxIter=int(1e3)):
     return gcf
 ###################################################################################
 def gini(g, b):
+    if len(g) == 0: return []
     if g.ndim == 1: g = g.reshape(len(g), 1); b = b.reshape(len(b), 1)
     [m, n] = g.shape
     G = g.sum(axis=0)
@@ -349,6 +354,7 @@ def gser(a, x):
     return gamser
 ###################################################################################
 def ks(g, b):
+    if len(g) == 0: return []
     if g.ndim == 1: g = g.reshape(len(g), 1); b = b.reshape(len(b), 1)
     [m, n] = g.shape
     G = g.sum(axis=0)
@@ -359,8 +365,6 @@ def ks(g, b):
     cpb = np.cumsum(pb, axis=0)
     ks = np.max(abs(cpg - cpb), axis=0)
     return ks
-
-
 ###################################################################################
 def pvalC(v, C2):
     return gamq(0.5 * v, 0.5 * C2)
@@ -420,6 +424,7 @@ def vif(x):
     return vifi
 ###################################################################################
 def vinf(g, b, tau=1e-12):
+    if len(g) == 0: return []
     G = sum(g)
     B = sum(b)
     gG = g / G
@@ -430,6 +435,7 @@ def vinf(g, b, tau=1e-12):
     return vinf
 ###################################################################################
 def woe(g, b, tau=1e-12):
+    if len(g) == 0: return []
     G = sum(g)
     B = sum(b)
     gG = g / G
