@@ -58,11 +58,6 @@ def dspl(flg, **kwargs):
             print('\n')
 ###################################################################################
 def f_exp(args):
-    # ---------------------------------------------
-    # Author: Alexander Efemov
-    # Date:   20.12.2009
-    # Course: Modelling and Processes Optimization
-    # ---------------------------------------------
     # F for exponential model: ym = a + b*ln(x)
     h = args['data'].shape[1]
     x = args['data'][:, np.arange(0, h - 3)]
@@ -82,12 +77,50 @@ def f_exp(args):
 
     return F, args
 ###################################################################################
+def f_exp1(args):
+   # F for exponential model: ym = a + b*exp(c*x)
+    h = args['data'].shape[1]
+    x = args['data'][:, np.arange(0, h - 3)]
+    y = c_(args['data'][:, -3])
+    w = c_(args['data'][:, -2])
+    pm = args['x']
+    if x.shape[1] == 0: x = c_(x)
+    ym = exp1_apl(x, pm)
+    e = y - ym
+    F = e.T@(e*w)
+    args['data'][:, -1] = ym.flatten()
+
+    b = 0.01
+    c = 1e6
+    dd = np.hstack((pm.flatten()[:2], zeros((1,))))
+    A = np.diag((dd < b))*c
+    F = F + pm.T@A@pm
+
+    return F, args
+###################################################################################
+def f_exp2(args):
+   # F for exponential model: ym = a + b*exp(c*x)
+    h = args['data'].shape[1]
+    x = args['data'][:, np.arange(0, h - 3)]
+    y = c_(args['data'][:, -3])
+    w = c_(args['data'][:, -2])
+    pm = args['x']
+    if x.shape[1] == 0: x = c_(x)
+    ym = exp2_apl(x, pm)
+    e = y - ym
+    F = e.T@(e*w)
+    args['data'][:, -1] = ym.flatten()
+
+    b = 0.01
+    c = 1e6
+    d1 = pm.flatten()
+    d2 = (1 - (pm[0, 0] + pm[1, 0]))**2*c
+    A = np.diag((d1 < b))*c
+    F = F + pm.T@A@pm + d2
+
+    return F, args
+###################################################################################
 def f_lgr(args):
-    # ---------------------------------------------
-    # Author: Alexander Efemov
-    # Date:   20.12.2009
-    # Course: Modelling and Processes Optimization
-    # ---------------------------------------------
     # F for logistic regression model: ym = 1/(1 + exp(-x*pm))
     h = args['data'].shape[1]
     x = args['data'][:, np.arange(0, h - 3)]
@@ -111,12 +144,6 @@ def f_lgr(args):
     return F, args
 ###################################################################################
 def f_log(args):
-    # ---------------------------------------------
-    # Author: Alexander Efemov
-    # Date:   20.12.2009
-    # Course: Modelling and Processes Optimization
-    # ---------------------------------------------
-
     # F for exponential model: ym = a + b*ln(x)
     h = args['data'].shape[1]
     x = args['data'][:, np.arange(0, h - 3)]
@@ -136,6 +163,26 @@ def f_log(args):
 
     return F, args
 ###################################################################################
+def f_log1(args):
+    # F for exponential model: ym = a + b*ln(x)
+    h = args['data'].shape[1]
+    x = args['data'][:, np.arange(0, h - 3)]
+    y = c_(args['data'][:, -3])
+    w = c_(args['data'][:, -2])
+    if x.shape[1] == 0: x = c_(x)
+    ym = log1_apl(x, args['x'])
+    e = y - ym
+    F = e.T@(e*w)
+    args['data'][:, -1] = ym.flatten()
+
+    pm = args['x']
+    b = 0.01
+    c = 1e6
+    A = np.diag((pm.flatten() < b))*c
+    F = F + pm.T@A@pm
+
+    return F, args
+###################################################################################
 def g_exp(args=None):
     # ---------------------------------------------
     # Author: Alexander Efemov
@@ -161,6 +208,61 @@ def g_exp(args=None):
 
     return g
 ###################################################################################
+def g_exp1(args=None):
+    # ---------------------------------------------
+    # Author: Alexander Efemov
+    # Date:   20.12.2009
+    # Course: Modelling and Processes Optimization
+    # ---------------------------------------------
+    # gradient of F for logistic regression model: ym = 1/(1 + exp(-x*pm))
+    h = args['data'].shape[1]
+    x = args['data'][:, np.arange(0, h - 3)]
+    y = c_(args['data'][:, -3])
+    w = c_(args['data'][:, -2])
+    ym = c_(args['data'][:, -1])
+    pm = args['x']
+    e = y - ym
+    tmp = np.exp(x + pm[2])
+    g = np.zeros((len(pm), 1))
+    g[0] = (-2*w.T@e)[0, 0]
+    g[1] = (-2*(w*e).T@tmp)[0]
+    g[2] = (-2*pm[1]*(w*e).T@tmp)[0]
+
+    b = 0.01
+    c = 1e6
+    dd = np.hstack((pm.flatten()[:2], zeros((1,))))
+    A = np.diag((dd < b))*c
+    g = g + 2*A@pm
+
+    return g
+###################################################################################
+def g_exp2(args=None):
+    # ---------------------------------------------
+    # Author: Alexander Efemov
+    # Date:   20.12.2009
+    # Course: Modelling and Processes Optimization
+    # ---------------------------------------------
+    # gradient of F for logistic regression model: ym = 1/(1 + exp(-x*pm))
+    h = args['data'].shape[1]
+    x = args['data'][:, np.arange(0, h - 3)]
+    y = c_(args['data'][:, -3])
+    w = c_(args['data'][:, -2])
+    ym = c_(args['data'][:, -1])
+    pm = args['x']
+    e = y - ym
+    tmp = np.exp(x*pm[2])
+    g = np.zeros((len(pm), 1))
+    g[0] = (-2*w.T@e)[0, 0]
+    g[1] = (-2*(w*e).T@tmp)[0]
+    g[2] = (-2*pm[1]*(w*e*x).T@tmp)[0]
+
+    b = 0.01
+    c = 1e6
+    dd = pm.flatten()
+    A = np.diag((dd < b))*c
+    g = g + 2*A@pm + np.array([[-2*(1 - (pm[0,0] + pm[1,0]))*c], [-2*(1 - (pm[0,0] + pm[1,0]))*c], [0]])
+    return g
+###################################################################################
 def g_lgr(args=None):
     # ---------------------------------------------
     # Author: Alexander Efemov
@@ -178,17 +280,12 @@ def g_lgr(args=None):
     # pm = args['x']
     # b = 0.01
     # c = 1e6
-    # A = np.diag((pm.flatten() < b)) * c
+    # A = np.diag((pm.flatten() < b))*c
     # g = g + 2*A@pm
     return g
 ###################################################################################
 def g_log(args=None):
-    # ---------------------------------------------
-    # Author: Alexander Efemov
-    # Date:   20.12.2009
-    # Course: Modelling and Processes Optimization
-    # ---------------------------------------------
-    # gradient of F for logistic regression model: ym = 1/(1 + exp(-x*pm))
+    # gradient of F for log model: ym = 1/(1 + exp(-x*pm))
     h = args['data'].shape[1]
     x = args['data'][:, np.arange(0, h - 3)]
     y = c_(args['data'][:, -3])
@@ -207,7 +304,7 @@ def g_log(args=None):
 
     return g
 ###################################################################################
-def H_exp(args=None):
+def h_exp(args=None):
     # ---------------------------------------------
     # Author: Alexander Efemov
     # Date:   20.12.2009
@@ -233,7 +330,77 @@ def H_exp(args=None):
 
     return H
 ###################################################################################
-def H_lgr(args=None):
+def h_exp1(args=None):
+    # ---------------------------------------------
+    # Author: Alexander Efemov
+    # Date:   20.12.2009
+    # Course: Modelling and Processes Optimization
+    # ---------------------------------------------
+    # Hessian of F for logistic regression model: ym = 1/(1 + exp(-x*pm))
+    h = args['data'].shape[1]
+    x = args['data'][:, np.arange(0, h - 3)]
+    w = c_(args['data'][:, -2])
+    y = c_(args['data'][:, -3])
+    ym = c_(args['data'][:, -1])
+    pm = args['x']
+    e = y - ym
+    H = np.zeros(shape=(3, 3))
+    tmp = np.exp(x + pm[2])
+    H[0, 0] = 2*np.sum(w)
+    H[0, 1] = 2*(w.T@tmp)[0]
+    H[0, 2] = 2*pm[1]*(w.T@tmp)[0]
+    H[1, 0] = H[0, 1]
+    H[1, 1] = 2*((w*tmp).T@tmp)[0]
+    H[1, 2] = 2*pm[1]*((w*tmp).T@tmp)[0] - 2*((w*e).T@tmp)[0]
+    H[2, 0] = H[0, 2]
+    H[2, 1] = H[1, 2]
+    H[2, 2] = 2*pm[1]**2*((w*tmp).T@tmp)[0] - 2*pm[1]*((w*e).T@tmp)[0]
+
+    pm = args['x']
+    b = 0.01
+    c = 1e6
+    dd = np.hstack((pm.flatten()[:2], zeros((1,))))
+    A = np.diag((dd < b))*c
+    H = H + 2*A
+    return H
+    dd = pm.flatten()
+    A = np.diag((dd < b))*c
+###################################################################################
+def h_exp2(args=None):
+    # ---------------------------------------------
+    # Author: Alexander Efemov
+    # Date:   20.12.2009
+    # Course: Modelling and Processes Optimization
+    # ---------------------------------------------
+    # Hessian of F for logistic regression model: ym = 1/(1 + exp(-x*pm))
+    h = args['data'].shape[1]
+    x = args['data'][:, np.arange(0, h - 3)]
+    w = c_(args['data'][:, -2])
+    y = c_(args['data'][:, -3])
+    ym = c_(args['data'][:, -1])
+    pm = args['x']
+    e = y - ym
+    H = np.zeros(shape=(3, 3))
+    tmp = np.exp(x*pm[2])
+    H[0, 0] = 2*np.sum(w)
+    H[0, 1] = 2*(w.T@tmp)[0]
+    H[0, 2] = 2*pm[1]*((w*x).T@tmp)[0]
+    H[1, 0] = H[0, 1]
+    H[1, 1] = 2*((w*tmp).T@tmp)[0]
+    H[1, 2] = 2*pm[1]*((w*tmp*x).T@tmp)[0] - 2*((w*e*x).T@tmp)[0]
+    H[2, 0] = H[0, 2]
+    H[2, 1] = H[1, 2]
+    H[2, 2] = 2*pm[1]**2*((w*tmp*x**2).T@tmp)[0] - 2*pm[1]*((w*e*x**2).T@tmp)[0]
+
+    pm = args['x']
+    b = 0.01
+    c = 1e6
+    dd = pm.flatten()
+    A = np.diag((dd < b))*c
+    H = H + 2*A + np.array([[2*c, 2*c, 0], [2*c, 2*c, 0], [0, 0, 0]])
+    return H
+###################################################################################
+def h_lgr(args=None):
     # ---------------------------------------------
     # Author: Alexander Efemov
     # Date:   20.12.2009
@@ -254,7 +421,7 @@ def H_lgr(args=None):
     # H = H + 2*A
     return H
 ###################################################################################
-def H_log(args=None):
+def h_log(args=None):
     # ---------------------------------------------
     # Author: Alexander Efemov
     # Date:   20.12.2009
@@ -283,6 +450,14 @@ def exp_apl(x, pm):
     ym = pm[0] + pm[1]*np.exp(x)
     return ym
 ###################################################################################
+def exp1_apl(x, pm):
+    ym = pm[0] + pm[1]*np.exp(x + pm[2])
+    return ym
+###################################################################################
+def exp2_apl(x, pm):
+    ym = pm[0] + pm[1]*np.exp(x*pm[2])
+    return ym
+###################################################################################
 def lgr_apl(x, pm):
     # logistic regression function
     if not isinstance(x, np.ndarray): x = np.array([[x]])
@@ -293,8 +468,10 @@ def lgr_apl(x, pm):
 def log_apl(x, pm):
     ym = pm[0] + pm[1]*np.log(abs(x) + 1)
     return ym
-
-
+###################################################################################
+def log1_apl(x, pm):
+    ym = pm[0] + pm[1]*np.log(pm[2]*abs(x) + 1)
+    return ym
 ###################################################################################
 def optOmit(x, xx, F, FF, g, g_1, H, H_1, Hinv, Hinv_1, iterate, s, tmp, sc, smeth, dsp_op):
     # ---------------------------------------------
@@ -338,8 +515,6 @@ def optOmit(x, xx, F, FF, g, g_1, H, H_1, Hinv, Hinv_1, iterate, s, tmp, sc, sme
                 msg = 'G'
             flg = 1
     return x, F, g, H, Hinv, tmp, sc, iterate, msg, flg
-
-
 ###################################################################################
 def stepPar(s, F, FF, g, g_1, iter, flg, sc, smeth, su, p_decr, p_incr):
     # Stepsize Determination
@@ -397,11 +572,9 @@ def stepPar(s, F, FF, g, g_1, iter, flg, sc, smeth, su, p_decr, p_incr):
     elif smeth == 'none':
         s = 1
     return s
-
-
 ###################################################################################
 def stopcrt(x=None, F=None, xx=None, FF=None, g=None, H=None, itr=None, maxiter=None, fcnv=None, afcnv=None, xcnv=None,
-            gcnv=None, sc=None, dsp_op=None):
+            gcnv=None, sc=None):
     # Stopping Criteria
     # Contains a set of convergence criteria for NR, NRLS, NRPI, SD, etc.
     # ---------------------------------------------
@@ -446,106 +619,100 @@ def stopcrt(x=None, F=None, xx=None, FF=None, g=None, H=None, itr=None, maxiter=
             iterate = 0
             msg = 'E'
             return iterate, msg
-    if gcnv is not None and g.T@H@g/(np.abs(F) + 1e-6) < gcnv:
+    if gcnv is not None and np.abs(g.T@H@g)/(np.abs(F) + 1e-6) < gcnv:
         iterate = 0
         msg = 'F'
         return iterate, msg
     return iterate, msg
 ###################################################################################
-
-
-import copy
-import numpy as np
-import pandas as pd
-
-
-def model_apply(mdl_type, X, pm):
-    if mdl_type == 'logistic':
-        return 1/(1 + np.exp(-np.dot(X, pm)))
-
-    if mdl_type == 'exp':
-        return (pm[0] + pm[1]*np.exp(X)).reshape((len(X), 1))
-
-    if mdl_type == 'log':
-        return (pm[0] + pm[1]*np.log(abs(X) + 1)).reshape((len(X), 1))
-
-    if mdl_type == 'logB':
-        pass
-        # return pm[0] + np.log(abs(x) + 1)/np.log(pm[2])
-
-
-def grad(mdl_type, ym, x, y, w):
-    if mdl_type == 'logistic':
-        return np.dot(-x.T, ((y - ym)*w))
-
-    else:
-        e = y - ym.reshape((len(y), 1))
-        g = np.zeros(shape=(2, 1))
-        g[0] = (-2*np.dot(w.T, e))[0, 0]
-
-    if mdl_type == 'exp':
-        g[1] = (-2*np.dot((w*e).T, np.exp(x)))[0]
-
-    if mdl_type == 'log':
-        g[1] = (-2*np.dot((w*e).T, np.log(abs(x) + 1)))[0]
-
-    if mdl_type == 'logB':
-        pass
-        # lnb = np.log(pm(2))
-        # g[1] = 2/(pm(2)*lnb)*np.dot((w*e).T, np.log(abs(x) + 1)/lnb)
-
-    return g
-
-
-def hes(mdl_type, ym, x, y, w):
-    if mdl_type == 'logistic':
-        return np.dot((x*np.tile(ym*(1 - ym)*w, (1, x.shape[1]))).T, x)
-
-    else:
-        H = np.zeros(shape=(2, 2))
-
-    if mdl_type == 'exp' or mdl_type == 'log':
-
-        if mdl_type == 'exp':
-            tmp = np.exp(x)
-
-        if mdl_type == 'log':
-            tmp = np.log(abs(x) + 1)
-
-        H[0, 0] = 2*w.sum()
-        H[0, 1] = (2*np.dot(w.T, tmp))[0]
-        H[1, 0] = H[0, 1]
-        H[1, 1] = (2*np.dot(w.T*tmp, tmp))[0]
-
-    if mdl_type == 'logB':
-        pass
-        # lnb = log(pm(2));
-        # logbc = log(abs(x) + 1)/log(pm(2));  % = logb(c) = logb(abs(x) + 1)
-        # H11 = 2*sum(w);
-        # H12 = -2/(pm(2)*lnb)*w'*logbc;
-        # H21 = H12;
-        # H22 = 2/(pm(2)*lnb)^2*((lnb + 3)*logbc' - (lnb + 2)*(y - pm(1))')*(w.*logbc);
-
-    return H
-
-
-def func(mdl_type, pm, x, y, w):
-    # Model Output
-    ym = model_apply(mdl_type, x, pm)
-
-    # Capping
-    ym[ym > 1 - 1e-10] = 1 - 1e-10
-    ym[ym < 1e-10] = 1e-10
-
-    if mdl_type == 'logistic':
-        yy = np.tile(y, (1, pm.shape[1]))
-        ww = np.tile(w, (1, pm.shape[1]))
-        F = -(yy*np.log(ym)*ww + (1 - yy)*np.log(1 - ym)*ww).sum().T
-
-    else:
-
-        ym = model_apply(mdl_type, x, pm)
-        e = y - ym
-        F = np.dot(e.T, (e*w))[0, 0]
-
-    return [F, ym]
+# import copy
+# import numpy as np
+# import pandas as pd
+# def model_apply(mdl_type, X, pm):
+#     if mdl_type == 'logistic':
+#         return 1/(1 + np.exp(-np.dot(X, pm)))
+#
+#     if mdl_type == 'exp':
+#         return (pm[0] + pm[1]*np.exp(X)).reshape((len(X), 1))
+#
+#     if mdl_type == 'log':
+#         return (pm[0] + pm[1]*np.log(abs(X) + 1)).reshape((len(X), 1))
+#
+#     if mdl_type == 'logB':
+#         pass
+#         # return pm[0] + np.log(abs(x) + 1)/np.log(pm[2])
+# def grad(mdl_type, ym, x, y, w):
+#     if mdl_type == 'logistic':
+#         return np.dot(-x.T, ((y - ym)*w))
+#
+#     else:
+#         e = y - ym.reshape((len(y), 1))
+#         g = np.zeros(shape=(2, 1))
+#         g[0] = (-2*np.dot(w.T, e))[0, 0]
+#
+#     if mdl_type == 'exp':
+#         g[1] = (-2*np.dot((w*e).T, np.exp(x)))[0]
+#
+#     if mdl_type == 'log':
+#         g[1] = (-2*np.dot((w*e).T, np.log(abs(x) + 1)))[0]
+#
+#     if mdl_type == 'logB':
+#         pass
+#         # lnb = np.log(pm(2))
+#         # g[1] = 2/(pm(2)*lnb)*np.dot((w*e).T, np.log(abs(x) + 1)/lnb)
+#
+#     return g
+#
+#
+# def hes(mdl_type, ym, x, y, w):
+#     if mdl_type == 'logistic':
+#         return np.dot((x*np.tile(ym*(1 - ym)*w, (1, x.shape[1]))).T, x)
+#
+#     else:
+#         H = np.zeros(shape=(2, 2))
+#
+#     if mdl_type == 'exp' or mdl_type == 'log':
+#
+#         if mdl_type == 'exp':
+#             tmp = np.exp(x)
+#
+#         if mdl_type == 'log':
+#             tmp = np.log(abs(x) + 1)
+#
+#         H[0, 0] = 2*w.sum()
+#         H[0, 1] = (2*np.dot(w.T, tmp))[0]
+#         H[1, 0] = H[0, 1]
+#         H[1, 1] = (2*np.dot(w.T*tmp, tmp))[0]
+#
+#     if mdl_type == 'logB':
+#         pass
+#         # lnb = log(pm(2));
+#         # logbc = log(abs(x) + 1)/log(pm(2));  % = logb(c) = logb(abs(x) + 1)
+#         # H11 = 2*sum(w);
+#         # H12 = -2/(pm(2)*lnb)*w'*logbc;
+#         # H21 = H12;
+#         # H22 = 2/(pm(2)*lnb)^2*((lnb + 3)*logbc' - (lnb + 2)*(y - pm(1))')*(w.*logbc);
+#
+#     return H
+#
+#
+# def func(mdl_type, pm, x, y, w):
+#     # Model Output
+#     ym = model_apply(mdl_type, x, pm)
+#
+#     # Capping
+#     ym[ym > 1 - 1e-10] = 1 - 1e-10
+#     ym[ym < 1e-10] = 1e-10
+#
+#     if mdl_type == 'logistic':
+#         yy = np.tile(y, (1, pm.shape[1]))
+#         ww = np.tile(w, (1, pm.shape[1]))
+#         F = -(yy*np.log(ym)*ww + (1 - yy)*np.log(1 - ym)*ww).sum().T
+#
+#     else:
+#
+#         ym = model_apply(mdl_type, x, pm)
+#         e = y - ym
+#         F = np.dot(e.T, (e*w))[0, 0]
+#
+#     return [F, ym]
